@@ -16,6 +16,7 @@ import mrcal
 
 
 def implied_Rt10__from_unprojections(q0, p0, v1,
+                                     *,
                                      weights      = None,
                                      atinfinity   = True,
                                      focus_center = np.zeros((2,), dtype=float),
@@ -161,9 +162,23 @@ report a full Rt transformation with the t component set to 0
 
     import scipy.optimize
 
+    ### flatten all the input arrays
+    # shape (N,2)
+    q0 = nps.clump(q0, n=q0.ndim-1)
+    # shape (M,N,3)
+    p0 = nps.transpose(nps.clump(nps.mv( nps.atleast_dims(p0, -3),
+                                         -1,-3),
+                                 n=-2))
+    # shape (N,3)
+    v1 = nps.clump(v1, n=v1.ndim-1)
+
     if weights is None:
         weights = np.ones(p0.shape[:-1], dtype=float)
     else:
+        # shape (..., Nh,Nw) -> (M,N,) where N = Nh*Nw
+        weights = nps.clump( nps.clump(weights, n=-2),
+                             n = weights.ndim-2)
+
         # Any inf/nan weight or vector are set to 0
         weights = weights.copy()
         weights[ ~np.isfinite(weights) ] = 0.0
@@ -171,10 +186,8 @@ report a full Rt transformation with the t component set to 0
     p0 = p0.copy()
     v1 = v1.copy()
 
-    # p0 had shape (..., Nh,Nw,3). Collapse all the leading dimensions into one
+    # p0 had shape (N,3). Collapse all the leading dimensions into one
     # And do the same for weights
-    p0      = nps.clump(p0,      n = len(p0.shape)     -3)
-    weights = nps.clump(weights, n = len(weights.shape)-2)
 
     i_nan_p0 = ~np.isfinite(p0)
     p0[i_nan_p0] = 0.
@@ -194,68 +207,69 @@ report a full Rt transformation with the t component set to 0
     if np.count_nonzero(i)<3:
         raise Exception("Focus region contained too few points")
 
-    p0_cut  = p0     [...,i, :]
-    v1_cut  = v1     [    i, :]
-    weights = weights[...,i   ]
+    p0_cut  = p0     [..., i, :]
+    v1_cut  = v1     [     i, :]
+    weights = weights[..., i   ]
 
     def residual_jacobian_rt(rt):
 
-        # rtp0 has shape (...,N,3)
+        # rtp0      has shape (M,N,3)
+        # drtp0_drt has shape (M,N,3,6)
         rtp0, drtp0_drt, _ = \
             mrcal.transform_point_rt(rt, p0_cut,
                                      get_gradients = True)
 
         # inner(a,b)/(mag(a)*mag(b)) = cos(x) ~ 1 - x^2/2
-        # Each of these has shape (...,N)
+        # Each of these has shape (M,N,)
         mag_rtp0 = nps.mag(rtp0)
         inner    = nps.inner(rtp0, v1_cut)
         th2      = 2.* (1.0 - inner / mag_rtp0)
         x        = th2 * weights
 
-        # shape (...,N,6)
-        dmag_rtp0_drt = nps.matmult( nps.dummy(rtp0, -2),   # shape (...,N,1,3)
-                                     drtp0_drt              # shape (...,N,3,6)
-                                     # matmult has shape (...,N,1,6)
+        # shape (M,N,6)
+        dmag_rtp0_drt = nps.matmult( nps.dummy(rtp0, -2),   # shape (M,N,1,3)
+                                     drtp0_drt              # shape (M,N,3,6)
+                                     # matmult has shape (M,N,1,6)
                                    )[...,0,:] / \
-                                   nps.dummy(mag_rtp0, -1)  # shape (...,N,1)
-        # shape (..., N,6)
-        dinner_drt    = nps.matmult( nps.dummy(v1_cut, -2), # shape (    N,1,3)
-                                     drtp0_drt              # shape (...,N,3,6)
-                                     # matmult has shape (...,N,1,6)
+                                   nps.dummy(mag_rtp0, -1)  # shape (M,N,1)
+        # shape (M,N,6)
+        dinner_drt    = nps.matmult( nps.dummy(v1_cut, -2), # shape (M,N,1,3)
+                                     drtp0_drt              # shape (M,N,3,6)
+                                     # matmult has shape (M,N,1,6)
                                    )[...,0,:]
 
         # dth2 = 2 (inner dmag_rtp0 - dinner mag_rtp0)/ mag_rtp0^2
-        # shape (...,N,6)
+        # shape (M,N,6)
         J = 2. * \
             (nps.dummy(inner,    -1) * dmag_rtp0_drt - \
              nps.dummy(mag_rtp0, -1) * dinner_drt) / \
              nps.dummy(mag_rtp0*mag_rtp0, -1) * \
              nps.dummy(weights,-1)
-        return x.ravel(), nps.clump(J, n=len(J.shape)-1)
+        return x.ravel(), nps.clump(J, n=J.ndim-1)
 
 
     def residual_jacobian_r(r):
 
-        # rp0     has shape (N,3)
-        # drp0_dr has shape (N,3,3)
+        # rp0     has shape (M,N,3)
+        # drp0_dr has shape (M,N,3,3)
         rp0, drp0_dr, _ = \
             mrcal.rotate_point_r(r, p0_cut,
                                  get_gradients = True)
 
         # inner(a,b)/(mag(a)*mag(b)) ~ cos(x) ~ 1 - x^2/2
-        # Each of these has shape (N)
+        # Each of these has shape (M,N)
         inner = nps.inner(rp0, v1_cut)
         th2   = 2.* (1.0 - inner)
         x     = th2 * weights
 
-        # shape (N,3)
-        dinner_dr = nps.matmult( nps.dummy(v1_cut, -2), # shape (N,1,3)
-                                 drp0_dr                # shape (N,3,3)
-                                 # matmult has shape (N,1,3)
-                               )[:,0,:]
+        # shape (M,N,3)
+        dinner_dr = nps.matmult( nps.dummy(v1_cut, -2), # shape (M,N,1,3)
+                                 drp0_dr                # shape (M,N,3,3)
+                                 # matmult has shape (M,N,1,3)
+                               )[...,0,:]
 
         J = -2. * dinner_dr * nps.dummy(weights,-1)
-        return x, J
+        return x.ravel(), nps.clump(J, n=J.ndim-1)
 
 
     cache = {'rt': None}
@@ -312,13 +326,12 @@ report a full Rt transformation with the t component set to 0
 
         # This is similar to a basic procrustes fit, but here we're using an L1
         # cost function
-
         r = np.random.random(3) * 1e-3
 
         res = scipy.optimize.least_squares(residual,
                                            r,
                                            jac=jacobian,
-                                           method='trf',
+                                           method='dogbox',
 
                                            loss=loss,
                                            f_scale = (f_scale_deg * np.pi/180.)**2.,
@@ -350,7 +363,7 @@ report a full Rt transformation with the t component set to 0
         res = scipy.optimize.least_squares(residual,
                                            rt,
                                            jac=jacobian,
-                                           method='trf',
+                                           method='dogbox',
 
                                            loss=loss,
                                            f_scale = (f_scale_deg * np.pi/180.)**2.,
@@ -442,76 +455,84 @@ broadcasting
     return np.sqrt((a+c)/2 + np.sqrt( (a-c)*(a-c)/4 + b*b))
 
 
-def _propagate_calibration_uncertainty( dF_dppacked,
+def _propagate_calibration_uncertainty( dF_dbpacked,
                                         factorization, Jpacked,
                                         Nmeasurements_observations,
                                         observed_pixel_uncertainty, what ):
     r'''Helper for uncertainty propagation functions
 
 Propagates the calibration-time uncertainty to compute Var(F) for some arbitrary
-vector F. The user specifies the gradient dF/dp: the sensitivity of F to noise
+vector F. The user specifies the gradient dF/db: the sensitivity of F to noise
 in the calibration state. The vector F can have any length: this is inferred
-from the dimensions of the given dF/dp gradient.
+from the dimensions of the given dF/db gradient.
 
-The given factorization uses the packed, unitless state: p*.
+The given factorization uses the packed, unitless state: b*.
 
-The given Jpacked uses the packed, unitless state: p*. Jpacked applies to all
+The given Jpacked uses the packed, unitless state: b*. Jpacked applies to all
 observations. The leading Nmeasurements_observations rows apply to the
 observations of the calibration object, and we use just those for the input
 noise propagation. if Nmeasurements_observations is None: assume that ALL the
 measurements come from the calibration object observations; a simplifed
 expression can be used in this case
 
-The given dF_dppacked uses the packed, unitless state p*, so it already includes
+The given dF_dbpacked uses the packed, unitless state b*, so it already includes
 the multiplication by D in the expressions below. It's usually sparse, but
 stored densely.
 
 The uncertainty computation in
 http://mrcal.secretsauce.net/uncertainty.html concludes that
 
-  Var(p*) = observed_pixel_uncertainty^2 inv(J*tJ*) J*[observations]t J*[observations] inv(J*tJ*)
+  Var(b) = observed_pixel_uncertainty^2 inv(JtJ) J[observations]t J[observations] inv(JtJ)
 
-where p* and J* are the UNITLESS state and the jacobian respectively.
+I actually operate with b* and J*: the UNITLESS state and the jacobian
+respectively. I have
 
-In the special case where all the measurements come from
-observations, this simplifies to
+  b = D b*
+  J = J* inv(D)
 
-  Var(p*) = observed_pixel_uncertainty^2 inv(J*tJ*)
+so
+
+  Var(b*) = observed_pixel_uncertainty^2 inv(J*tJ*) J*[observations]t J*[observations] inv(J*tJ*)
+
+In the special case where all the measurements come from observations, this
+simplifies to
+
+  Var(b*) = observed_pixel_uncertainty^2 inv(J*tJ*)
 
 My factorization is of packed (scaled, unitless) flavors of J (J*). So
 
-  Var(p) = D Var(p*) D
+  Var(b) = D Var(b*) D
 
-I want Var(F) = dF/dp Var(p) dF/dpt
+I want Var(F) = dF/db Var(b) dF/dbt
 
 So
 
-  Var(F) = dF/dp D Var(p*) D dF/dpt
+  Var(F) = dF/db D Var(b*) D dF/dbt
 
 In the regularized case I have
 
-  Var(F) = dF/dp D inv(J*tJ*) J*[observations]t J*[observations] inv(J*tJ*) D dF/dpt observed_pixel_uncertainty^2
+  Var(F) = dF/db D inv(J*tJ*) J*[observations]t J*[observations] inv(J*tJ*) D dF/dbt observed_pixel_uncertainty^2
 
-It is far more efficient to compute inv(J*tJ*) D dF/dpt than
+It is far more efficient to compute inv(J*tJ*) D dF/dbt than
 inv(J*tJ*) J*[observations]t: there's far less to compute, and the matrices
 are far smaller. Thus I don't compute the covariances directly.
 
 In the non-regularized case:
 
-  Var(F) = dF/dp D inv(J*tJ*) D dF/dpt
+  Var(F) = dF/db D inv(J*tJ*) D dF/dbt
 
-  1. solve( J*tJ*, D dF/dpt)
+  1. solve( J*tJ*, D dF/dbt)
      The result has shape (Nstate,2)
 
-  2. pre-multiply by dF/dp D
+  2. pre-multiply by dF/db D
 
   3. multiply by observed_pixel_uncertainty^2
 
 In the regularized case:
 
-  Var(F) = dF/dp D inv(J*tJ*) J*[observations]t J*[observations] inv(J*tJ*) D dF/dpt
+  Var(F) = dF/db D inv(J*tJ*) J*[observations]t J*[observations] inv(J*tJ*) D dF/dbt
 
-  1. solve( J*tJ*, D dF/dpt)
+  1. solve( J*tJ*, D dF/dbt)
      The result has shape (Nstate,2)
 
   2. Pre-multiply by J*[observations]
@@ -524,7 +545,7 @@ In the regularized case:
     '''
 
     # shape (2,Nstate)
-    A = factorization.solve_xt_JtJ_bt( dF_dppacked )
+    A = factorization.solve_xt_JtJ_bt( dF_dbpacked )
     if Nmeasurements_observations is not None:
         # I have regularization. Use the more complicated expression
 
@@ -535,7 +556,7 @@ In the regularized case:
                                               Nleading_rows_J = Nmeasurements_observations)
     else:
         # No regularization. Use the simplified expression
-        Var_dF = nps.matmult(dF_dppacked, nps.transpose(A))
+        Var_dF = nps.matmult(dF_dbpacked, nps.transpose(A))
 
     if what == 'covariance':           return Var_dF * observed_pixel_uncertainty*observed_pixel_uncertainty
     if what == 'worstdirection-stdev': return worst_direction_stdev(Var_dF) * observed_pixel_uncertainty
@@ -562,7 +583,7 @@ def _projection_uncertainty( p_cam,
     '''
 
     Nstate = Jpacked.shape[-1]
-    dq_dpief = np.zeros(p_cam.shape[:-1] + (2,Nstate), dtype=float)
+    dq_dbief = np.zeros(p_cam.shape[:-1] + (2,Nstate), dtype=float)
 
     if frames_rt_toref is not None:
         Nframes = len(frames_rt_toref)
@@ -611,7 +632,7 @@ def _projection_uncertainty( p_cam,
     if istate_intrinsics is not None:
         dq_dintrinsics_optimized = dq_dintrinsics[..., slice_optimized_intrinsics]
         Nintrinsics = dq_dintrinsics_optimized.shape[-1]
-        dq_dpief[..., istate_intrinsics:istate_intrinsics+Nintrinsics] = \
+        dq_dbief[..., istate_intrinsics:istate_intrinsics+Nintrinsics] = \
             dq_dintrinsics_optimized
 
     if extrinsics_rt_fromref is not None:
@@ -619,22 +640,22 @@ def _projection_uncertainty( p_cam,
             mrcal.transform_point_rt(extrinsics_rt_fromref, p_ref,
                                      get_gradients = True)
 
-        dq_dpief[..., istate_extrinsics:istate_extrinsics+6] = \
+        dq_dbief[..., istate_extrinsics:istate_extrinsics+6] = \
             nps.matmult(dq_dpcam, dpcam_drt)
 
         if frames_rt_toref is not None:
-            dq_dpief[..., istate_frames:istate_frames+Nframes*6] = \
+            dq_dbief[..., istate_frames:istate_frames+Nframes*6] = \
                 nps.matmult(dq_dpcam, dpcam_dpref, dpref_dframes)
     else:
         if frames_rt_toref is not None:
-            dq_dpief[..., istate_frames:istate_frames+Nframes*6] = \
+            dq_dbief[..., istate_frames:istate_frames+Nframes*6] = \
                 nps.matmult(dq_dpcam, dpref_dframes)
 
-    # Make dq_dpief use the packed state. I call "unpack_state" because the
+    # Make dq_dbief use the packed state. I call "unpack_state" because the
     # state is in the denominator
-    mrcal.unpack_state(dq_dpief, **optimization_inputs)
+    mrcal.unpack_state(dq_dbief, **optimization_inputs)
     return \
-        _propagate_calibration_uncertainty( dq_dpief,
+        _propagate_calibration_uncertainty( dq_dbief,
                                             factorization, Jpacked,
                                             Nmeasurements_observations,
                                             observed_pixel_uncertainty,
@@ -660,7 +681,7 @@ def _projection_uncertainty_rotationonly( p_cam,
     '''
 
     Nstate = Jpacked.shape[-1]
-    dq_dpief = np.zeros(p_cam.shape[:-1] + (2,Nstate), dtype=float)
+    dq_dbief = np.zeros(p_cam.shape[:-1] + (2,Nstate), dtype=float)
 
     if frames_rt_toref is not None:
         Nframes = len(frames_rt_toref)
@@ -703,14 +724,14 @@ def _projection_uncertainty_rotationonly( p_cam,
     if istate_intrinsics is not None:
         dq_dintrinsics_optimized = dq_dintrinsics[..., slice_optimized_intrinsics]
         Nintrinsics = dq_dintrinsics_optimized.shape[-1]
-        dq_dpief[..., istate_intrinsics:istate_intrinsics+Nintrinsics] = \
+        dq_dbief[..., istate_intrinsics:istate_intrinsics+Nintrinsics] = \
             dq_dintrinsics_optimized
 
     if extrinsics_rt_fromref is not None:
         _, dpcam_dr, dpcam_dpref = \
             mrcal.rotate_point_r(extrinsics_rt_fromref[...,:3], p_ref,
                                  get_gradients = True)
-        dq_dpief[..., istate_extrinsics:istate_extrinsics+3] = \
+        dq_dbief[..., istate_extrinsics:istate_extrinsics+3] = \
             nps.matmult(dq_dpcam, dpcam_dr)
 
         if frames_rt_toref is not None:
@@ -719,20 +740,20 @@ def _projection_uncertainty_rotationonly( p_cam,
 
             # dprefallframes_dframesr has shape (..., Nframes,3,3)
             for i in range(Nframes):
-                dq_dpief[..., istate_frames+6*i:istate_frames+6*i+3] = \
+                dq_dbief[..., istate_frames+6*i:istate_frames+6*i+3] = \
                     nps.matmult(dq_dpref, dprefallframes_dframesr[...,i,:,:]) / Nframes
     else:
         if frames_rt_toref is not None:
             # dprefallframes_dframesr has shape (..., Nframes,3,3)
             for i in range(Nframes):
-                dq_dpief[..., istate_frames+6*i:istate_frames+6*i+3] = \
+                dq_dbief[..., istate_frames+6*i:istate_frames+6*i+3] = \
                     nps.matmult(dq_dpcam, dprefallframes_dframesr[...,i,:,:]) / Nframes
 
-    # Make dq_dpief use the packed state. I call "unpack_state" because the
+    # Make dq_dbief use the packed state. I call "unpack_state" because the
     # state is in the denominator
-    mrcal.unpack_state(dq_dpief, **optimization_inputs)
+    mrcal.unpack_state(dq_dbief, **optimization_inputs)
     return \
-        _propagate_calibration_uncertainty( dq_dpief,
+        _propagate_calibration_uncertainty( dq_dbief,
                                             factorization, Jpacked,
                                             Nmeasurements_observations,
                                             observed_pixel_uncertainty,
@@ -740,6 +761,7 @@ def _projection_uncertainty_rotationonly( p_cam,
 
 
 def projection_uncertainty( p_cam, model,
+                            *,
                             atinfinity = False,
 
                             # what we're reporting
@@ -834,39 +856,39 @@ else:                    we return an array of shape (...)
     '''
 
 
-    # I computed Var(p) earlier, which contains the variance of ALL the optimization
+    # I computed Var(b) earlier, which contains the variance of ALL the optimization
     # parameters together. The noise on the chessboard poses is coupled to the noise
     # on the extrinsics and to the noise on the intrinsics. And we can apply all these
     # together to propagate the uncertainty.
 
     # Let's define some variables:
 
-    # - p_i: the intrinsics of a camera
-    # - p_e: the extrinsics of that camera (T_cr)
-    # - p_f: ALL the chessboard poses (T_fr)
-    # - p_ief: the concatenation of p_i, p_e and p_f
+    # - b_i: the intrinsics of a camera
+    # - b_e: the extrinsics of that camera (T_cr)
+    # - b_f: ALL the chessboard poses (T_fr)
+    # - b_ief: the concatenation of b_i, b_e and b_f
 
     # I have
 
-    #     dq = q0 + dq/dp_ief dp_ief
+    #     dq = q0 + dq/db_ief db_ief
 
-    #     Var(q) = dq/dp_ief Var(p_ief) (dq/dp_ief)t
+    #     Var(q) = dq/db_ief Var(b_ief) (dq/db_ief)t
 
-    #     Var(p_ief) is a subset of Var(p), computed above.
+    #     Var(b_ief) is a subset of Var(b), computed above.
 
-    #     dq/dp_ief = [dq/dp_i dq/dp_e dq/dp_f]
+    #     dq/db_ief = [dq/db_i dq/db_e dq/db_f]
 
-    #     dq/dp_e = dq/dpcam dpcam/dp_e
+    #     dq/db_e = dq/dpcam dpcam/db_e
 
-    #     dq/dp_f = dq/dpcam dpcam/dpref dpref/dp_f / Nframes
+    #     dq/db_f = dq/dpcam dpcam/dpref dpref/db_f / Nframes
 
-    # dq/dp_i and all the constituent expressions comes directly from the project()
+    # dq/db_i and all the constituent expressions comes directly from the project()
     # and transform calls above. Depending on the details of the optimization problem,
     # some of these may not exist. For instance, if we're looking at a camera that is
-    # sitting at the reference coordinate system, then there is no p_e, and Var_ief is
+    # sitting at the reference coordinate system, then there is no b_e, and Var_ief is
     # smaller: it's just Var_if. If we somehow know the poses of the frames, then
     # there's no Var_f. If we want to know the uncertainty at distance=infinity, then
-    # we ignore all the translation components of p_e and p_f.
+    # we ignore all the translation components of b_e and b_f.
 
 
 
@@ -906,7 +928,7 @@ else:                    we return an array of shape (...)
     if not optimization_inputs.get('do_optimize_extrinsics'):
         raise Exception("Computing uncertainty if !do_optimize_extrinsics not supported currently. This is possible, but not implemented. _projection_uncertainty...() would need a path for fixed extrinsics like they already do for fixed frames")
 
-    ppacked,x,Jpacked,factorization = \
+    bpacked,x,Jpacked,factorization = \
         mrcal.optimizer_callback( **optimization_inputs )
 
     if factorization is None:
@@ -997,6 +1019,8 @@ else:                    we return an array of shape (...)
 
 
 def projection_diff(models,
+                    *,
+                    implied_Rt10 = None,
                     gridn_width  = 60,
                     gridn_height = None,
 
@@ -1033,7 +1057,7 @@ instance, one may want to validate a calibration by comparing the results of two
 different chessboard dances. Or one may want to evaluate the stability of the
 intrinsics in response to mechanical or thermal stresses. This function makes
 these comparisons, and returns the results. A visualization wrapper is
-available: mrcal.show_projection_diff()
+available: mrcal.show_projection_diff() and the mrcal-show-projection-diff tool.
 
 In the most common case we're given exactly 2 models to compare, and we compute
 the differences in projection of each point. If we're given more than 2 models,
@@ -1049,7 +1073,9 @@ The top-level operation of this function:
 - Project the transformed points to the other camera
 - Look at the resulting pixel difference in the reprojection
 
-Several variables control how we obtain the transformation. Top-level logic:
+If implied_Rt10 is given, we simply use that as the transformation (this is
+currently supported ONLY for diffing exactly 2 cameras). If implied_Rt10 is not
+given, we estimate it. Several variables control this. Top-level logic:
 
   if intrinsics_only:
       Rt10 = identity_Rt()
@@ -1118,7 +1144,7 @@ is specified by the "distance" argument. By default (distance = None) we look
 out to infinity. If we care about the projection difference at some other
 distance, pass that here. Multiple distances can be passed in an iterable. We'll
 then fit the implied-by-the-intrinsics transformation using all the distances,
-and we'll display the best-fitting difference for each pixel. Generally the most
+and we'll return the differences for each given distance. Generally the most
 confident distance will be where the chessboards were observed at calibration
 time.
 
@@ -1128,6 +1154,11 @@ ARGUMENTS
   will be 2 of these, but more than 2 is allowed. The intrinsics are always
   used; the extrinsics are used only if not intrinsics_only and focus_radius==0
 
+- implied_Rt10: optional transformation to use to line up the camera coordinate
+  systems. Most of the time we want to estimate this transformation, so this
+  should be omitted or None. Currently this is supported only if exactly two
+  models are being compared.
+
 - gridn_width: optional value, defaulting to 60. How many points along the
   horizontal gridding dimension
 
@@ -1174,17 +1205,19 @@ RETURNED VALUE
 
 A tuple
 
-- difflen: a numpy array of shape (gridn_height,gridn_width) containing the
+- difflen: a numpy array of shape (...,gridn_height,gridn_width) containing the
   magnitude of differences at each cell, or the standard deviation of the
   differences between models 1..N and model0 if len(models)>2. if
-  len(models)==2: this is nps.mag(diff)
+  len(models)==2: this is nps.mag(diff). If the given 'distance' argument was an
+  iterable, the shape is (len(distance),...). Otherwise the shape is (...)
 
-- diff: a numpy array of shape (gridn_height,gridn_width,2) containing the
+- diff: a numpy array of shape (...,gridn_height,gridn_width,2) containing the
   vector of differences at each cell. If len(models)>2 this isn't defined, so
-  None is returned
+  None is returned. If the given 'distance' argument was an iterable, the shape
+  is (len(distance),...). Otherwise the shape is (...)
 
-- q0: a numpy array of shape (gridn_height,gridn_width,2) containing the pixel
-  coordinates of each grid cell
+- q0: a numpy array of shape (gridn_height,gridn_width,2) containing the
+  pixel coordinates of each grid cell
 
 - Rt10: the geometric Rt transformation in an array of shape (...,4,3). This is
   the relative transformation we ended up using, which is computed using the
@@ -1196,14 +1229,24 @@ A tuple
 
     if len(models) < 2:
         raise Exception("At least 2 models are required to compute the diff")
+    if len(models) > 2 and implied_Rt10 is not None:
+        raise Exception("A given implied_Rt10 is currently supported ONLY if exactly 2 models are being compared")
+
+    # If the distance is iterable, the shape of the output is (len(distance),
+    # ...). Otherwise it is just (...). In the intermediate computations, the
+    # quantities ALWAYS have shape (len(distance), ...). I select the desired
+    # shape at the end
+    distance_is_iterable = True
+    try:    len(distance)
+    except: distance_is_iterable = False
 
     if distance is None:
         atinfinity = True
-        distance   = 1.0
+        distance   = np.ones((1,), dtype=float)
     else:
         atinfinity = False
         distance   = nps.atleast_dims(np.array(distance), -1)
-        distance   = nps.mv(distance.ravel(), -1,-4)
+    distance   = nps.mv(distance.ravel(), -1,-4)
 
     imagersizes = np.array([model.imagersize() for model in models])
     if np.linalg.norm(np.std(imagersizes, axis=-2)) != 0:
@@ -1222,12 +1265,14 @@ A tuple
                                          normalize = True)
 
     uncertainties = None
-    if not intrinsics_only and focus_radius != 0 and \
-       use_uncertainties:
+    if use_uncertainties and \
+       not intrinsics_only and focus_radius != 0 and \
+       implied_Rt10 is None:
         try:
             # len(uncertainties) = Ncameras. Each has shape (len(distance),Nh,Nw)
             uncertainties = \
-                [ mrcal.projection_uncertainty(v[i] * distance,
+                [ mrcal.projection_uncertainty(# shape (len(distance),Nheight,Nwidth,3)
+                                               v[i] * distance,
                                                models[i],
                                                atinfinity = atinfinity,
                                                what       = 'worstdirection-stdev') \
@@ -1247,38 +1292,46 @@ A tuple
 
     if len(models) == 2:
         # Two models. Take the difference and call it good
-        if intrinsics_only:
-            Rt10 = mrcal.identity_Rt()
+        if implied_Rt10 is not None:
+            Rt10 = implied_Rt10
+
         else:
-            if focus_radius == 0:
-                Rt10 = mrcal.compose_Rt(models[1].extrinsics_Rt_fromref(),
-                                        models[0].extrinsics_Rt_toref())
+            if intrinsics_only:
+                Rt10 = mrcal.identity_Rt()
             else:
-                # weights has shape (len(distance),Nh,Nw))
-                if uncertainties is not None:
-                    weights = 1.0 / (uncertainties[0]*uncertainties[1])
-
-                    # It appears to work better if I discount the uncertain regions
-                    # even more. This isn't a principled decision, and is supported
-                    # only by a little bit of data. The differencing.org I'm writing
-                    # now will contain a weighted diff of culled and not-culled
-                    # splined model data. That diff computation requires this.
-                    weights *= weights
+                if focus_radius == 0:
+                    Rt10 = mrcal.compose_Rt(models[1].extrinsics_Rt_fromref(),
+                                            models[0].extrinsics_Rt_toref())
                 else:
-                    weights = None
+                    # weights has shape (len(distance),Nh,Nw))
+                    if uncertainties is not None:
+                        weights = 1.0 / (uncertainties[0]*uncertainties[1])
+
+                        # It appears to work better if I discount the uncertain regions
+                        # even more. This isn't a principled decision, and is supported
+                        # only by a little bit of data. The differencing.org I'm writing
+                        # now will contain a weighted diff of culled and not-culled
+                        # splined model data. That diff computation requires this.
+                        weights *= weights
+                    else:
+                        weights = None
+
+                    # weight may be inf or nan. implied_Rt10__from_unprojections() will
+                    # clean those up, as well as any inf/nan in v (from failed
+                    # unprojections)
+                    Rt10 = \
+                        implied_Rt10__from_unprojections(q0,
+                                                         # shape (len(distance),Nheight,Nwidth,3)
+                                                         v[0,...] * distance,
+                                                         v[1,...],
+                                                         weights      = weights,
+                                                         atinfinity   = atinfinity,
+                                                         focus_center = focus_center,
+                                                         focus_radius = focus_radius)
 
-                # weight may be inf or nan. implied_Rt10__from_unprojections() will
-                # clean those up, as well as any inf/nan in v (from failed
-                # unprojections)
-                Rt10 = \
-                    implied_Rt10__from_unprojections(q0,
-                                                     v[0,...] * distance,
-                                                     v[1,...],
-                                                     weights,
-                                                     atinfinity,
-                                                     focus_center, focus_radius)
 
         q1 = mrcal.project( mrcal.transform_point_Rt(Rt10,
+                                                     # shape (len(distance),Nheight,Nwidth,3)
                                                      v[0,...] * distance),
                             lensmodels[1], intrinsics_data[1])
         # shape (len(distance),Nheight,Nwidth,2)
@@ -1286,7 +1339,7 @@ A tuple
 
         diff    = q1 - q0
         difflen = nps.mag(diff)
-        difflen = np.min( difflen, axis=-3)
+
     else:
 
         # Many models. Look at the stdev
@@ -1316,27 +1369,47 @@ A tuple
             v1 = v[i1,...]
 
             return \
-                implied_Rt10__from_unprojections(q0, v0*distance, v1,
-                                                 weights, atinfinity,
-                                                 focus_center, focus_radius)
+                implied_Rt10__from_unprojections(q0,
+                                                 # shape (len(distance),Nheight,Nwidth,3)
+                                                 v0*distance,
+                                                 v1,
+                                                 weights      = weights,
+                                                 atinfinity   = atinfinity,
+                                                 focus_center = focus_center,
+                                                 focus_radius = focus_radius)
         def get_reprojections(q0, Rt10,
                               lensmodel, intrinsics_data):
-            q1 = mrcal.project(mrcal.transform_point_Rt(Rt10, v[0,...]*distance),
+            q1 = mrcal.project(mrcal.transform_point_Rt(Rt10,
+                                                        # shape (len(distance),Nheight,Nwidth,3)
+                                                        v[0,...]*distance),
                                lensmodel, intrinsics_data)
             # returning shape (len(distance),Nheight,Nwidth,2)
             return nps.atleast_dims(q1, -4)
 
-        Rt10 = nps.cat(*[ get_Rt10(0,i) for i in range(1,len(v))])
+        Ncameras = len(v)
+        # shape (Ncameras-1, 4,3)
+        Rt10 = nps.cat(*[ get_Rt10(0,i) for i in range(1,Ncameras)])
 
         # shape (Ncameras-1,len(distance),Nheight,Nwidth,2)
         grids = nps.cat(*[get_reprojections(q0, Rt10[i-1],
                                             lensmodels[i], intrinsics_data[i]) \
-                          for i in range(1,len(v))])
+                          for i in range(1,Ncameras)])
 
         diff    = None
-        difflen = np.sqrt(np.mean( np.min(nps.norm2(grids-q0),
-                                          axis=-3),
-                                   axis=0))
+        # shape (len(distance),Nheight,Nwidth)
+        difflen = np.sqrt(np.mean(nps.norm2(grids-q0),axis=0))
+
+    # difflen, diff, q0 currently all have shape (len(distance), ...). If the
+    # given distance was NOT an iterable, I strip out that leading dimension
+    if not distance_is_iterable:
+        if difflen.shape[0] != 1:
+            raise Exception(f"distance_is_iterable is False, but leading shape of difflen is not 1 (difflen.shape = {difflen.shape}). This is a bug. Giving up")
+        difflen = difflen[0,...]
+
+        if diff is not None:
+            if diff.shape[0] != 1:
+                raise Exception(f"distance_is_iterable is False, but leading shape of diff is not 1 (diff.shape = {diff.shape}). This is a bug. Giving up")
+            diff = diff[0,...]
 
     return difflen, diff, q0, Rt10
 
