@@ -12,13 +12,15 @@ import numpysane as nps
 import mrcal
 
 def rectified_system(models,
+                     *,
                      az_fov_deg,
                      el_fov_deg,
                      az0_deg             = None,
                      el0_deg             = 0,
                      pixels_per_deg_az   = -1.,
                      pixels_per_deg_el   = -1.,
-                     rectification_model = 'LENSMODEL_LATLON'):
+                     rectification_model = 'LENSMODEL_LATLON',
+                     return_metadata     = False):
 
     r'''Build rectified models for stereo rectification
 
@@ -34,7 +36,7 @@ SYNOPSIS
                for f in ('left.cameramodel',
                          'right.cameramodel') ]
 
-    images = [ cv2.imread(f) \
+    images = [ mrcal.load_image(f) \
                for f in ('left.jpg', 'right.jpg') ]
 
     models_rectified = \
@@ -63,26 +65,13 @@ SYNOPSIS
                               speckleRange      = 1)
     disparity16 = matcher.compute(*images_rectified) # in pixels*16
 
-    # Convert the disparities to range-to-camera0
-    ranges = mrcal.stereo_range( disparity16,
-                                 models_rectified,
-                                 disparity_scale = 16 )
-
-    H,W = disparity16.shape
-
-    # shape (H,W,2)
-    q = np.ascontiguousarray( \
-           nps.mv( nps.cat( *np.meshgrid(np.arange(W,dtype=float),
-                                         np.arange(H,dtype=float))),
-                   0, -1))
-
     # Point cloud in rectified camera-0 coordinates
     # shape (H,W,3)
-    p_rect0 = \
-        mrcal.unproject_latlon(q, models_rectified[0].intrinsics()[1]) * \
-        nps.dummy(ranges, axis=-1)
+    p_rect0 = mrcal.stereo_unproject( disparity16,
+                                      models_rectified,
+                                      disparity_scale = 16 )
 
-    Rt_cam0_rect0 = mrcal.compose_Rt( models[0].extrinsics_Rt_fromref(),
+    Rt_cam0_rect0 = mrcal.compose_Rt( models          [0].extrinsics_Rt_fromref(),
                                       models_rectified[0].extrinsics_Rt_toref() )
 
     # Point cloud in camera-0 coordinates
@@ -162,7 +151,8 @@ ARGUMENTS
   That causes the baseline to no longer be perpendicular with the view axis of
   the cameras, and thus the azimuth=0 vector no longer points "forward". If
   omitted, we compute az0_deg to align the center of the rectified system with
-  the center of the two cameras' views
+  the center of the two cameras' views. This computed value can be retrieved in
+  the metadata dict by passing return_metadata = True
 
 - el0_deg: optional value for the elevation center of the rectified system.
   Defaults to 0.
@@ -170,9 +160,10 @@ ARGUMENTS
 - pixels_per_deg_az: optional value for the azimuth resolution of the rectified
   image. If a resultion of >0 is requested, the value is used as is. If a
   resolution of <0 is requested, we use this as a scale factor on the resolution
-  of the input image. For instance, to downsample by a factor of 2, pass
+  of the first input image. For instance, to downsample by a factor of 2, pass
   pixels_per_deg_az = -0.5. By default, we use -1: the resolution of the input
-  image at the center of the rectified system.
+  image at the center of the rectified system. The value we end up with can be
+  retrieved in the metadata dict by passing return_metadata = True
 
 - pixels_per_deg_el: same as pixels_per_deg_az but in the elevation direction
 
@@ -181,12 +172,28 @@ ARGUMENTS
   lens model. Currently supported are "LENSMODEL_LATLON" (the default) and
   "LENSMODEL_PINHOLE"
 
+- return_metadata: optional boolean, defaulting to False. If True, we return a
+  dict of metadata describing the rectified system in addition to the rectified
+  models. This is useful to retrieve any of the autodetected values. At this
+  time, the metadata dict contains keys:
+
+    - az_fov_deg
+    - el_fov_deg
+    - az0_deg
+    - el0_deg
+    - pixels_per_deg_az
+    - pixels_per_deg_el
+    - baseline
+
 RETURNED VALUES
 
-We return a tuple of mrcal.cameramodels describing the two rectified cameras.
+We compute a tuple of mrcal.cameramodels describing the two rectified cameras.
 These two models are identical, except for a baseline translation in the +x
 direction in rectified coordinates.
 
+if not return_metadata: we return this tuple of models
+else:                   we return this tuple of models, dict of metadata
+
     '''
 
     if not (rectification_model == 'LENSMODEL_LATLON' or \
@@ -468,6 +475,9 @@ direction in rectified coordinates.
     else:
         raise Exception("Shouldn't get here; This case was checked above")
 
+    if Nel <= 0:
+        raise Exception(f"Resulting stereo geometry has Nel={Nel}. This is nonsensical. You should examine the geometry or adjust the elevation bounds or pixels-per-deg")
+
     ######## The geometry
     Rt_rect0_cam0 = nps.glue(R_rect0_cam0, np.zeros((3,),), axis=-2)
     Rt_rect0_ref  = mrcal.compose_Rt( Rt_rect0_cam0,
@@ -487,7 +497,20 @@ direction in rectified coordinates.
                              imagersize = (Naz, Nel),
                              extrinsics_Rt_fromref = Rt_rect1_ref) )
 
-    return models_rectified
+    if not return_metadata:
+        return models_rectified
+
+    metadata = \
+        dict( az_fov_deg        = az_fov_deg,
+              el_fov_deg        = el_fov_deg,
+              az0_deg           = az0 * 180./np.pi,
+              el0_deg           = el0_deg,
+              pixels_per_deg_az = pixels_per_deg_az,
+              pixels_per_deg_el = pixels_per_deg_el,
+              baseline          = baseline )
+
+    return models_rectified, metadata
+
 
 
 def _validate_models_rectified(models_rectified):
@@ -545,7 +568,7 @@ SYNOPSIS
                for f in ('left.cameramodel',
                          'right.cameramodel') ]
 
-    images = [ cv2.imread(f) \
+    images = [ mrcal.load_image(f) \
                for f in ('left.jpg', 'right.jpg') ]
 
     models_rectified = \
@@ -574,26 +597,13 @@ SYNOPSIS
                               speckleRange      = 1)
     disparity16 = matcher.compute(*images_rectified) # in pixels*16
 
-    # Convert the disparities to range-to-camera0
-    ranges = mrcal.stereo_range( disparity16,
-                                 models_rectified,
-                                 disparity_scale = 16 )
-
-    H,W = disparity16.shape
-
-    # shape (H,W,2)
-    q = np.ascontiguousarray( \
-           nps.mv( nps.cat( *np.meshgrid(np.arange(W,dtype=float),
-                                         np.arange(H,dtype=float))),
-                   0, -1))
-
     # Point cloud in rectified camera-0 coordinates
     # shape (H,W,3)
-    p_rect0 = \
-        mrcal.unproject_latlon(q, models_rectified[0].intrinsics()[1]) * \
-        nps.dummy(ranges, axis=-1)
+    p_rect0 = mrcal.stereo_unproject( disparity16,
+                                      models_rectified,
+                                      disparity_scale = 16 )
 
-    Rt_cam0_rect0 = mrcal.compose_Rt( models[0].extrinsics_Rt_fromref(),
+    Rt_cam0_rect0 = mrcal.compose_Rt( models          [0].extrinsics_Rt_fromref(),
                                       models_rectified[0].extrinsics_Rt_toref() )
 
     # Point cloud in camera-0 coordinates
@@ -667,6 +677,7 @@ contains corresponding pixel coordinates in the input image
 
 def stereo_range(disparity,
                  models_rectified,
+                 *,
                  disparity_scale = 1,
                  qrect0          = None):
 
@@ -684,7 +695,7 @@ SYNOPSIS
                for f in ('left.cameramodel',
                          'right.cameramodel') ]
 
-    images = [ cv2.imread(f) \
+    images = [ mrcal.load_image(f) \
                for f in ('left.jpg', 'right.jpg') ]
 
     models_rectified = \
@@ -732,7 +743,7 @@ SYNOPSIS
         mrcal.unproject_latlon(q, models_rectified[0].intrinsics()[1]) * \
         nps.dummy(ranges, axis=-1)
 
-    Rt_cam0_rect0 = mrcal.compose_Rt( models[0].extrinsics_Rt_fromref(),
+    Rt_cam0_rect0 = mrcal.compose_Rt( models          [0].extrinsics_Rt_fromref(),
                                       models_rectified[0].extrinsics_Rt_toref() )
 
     # Point cloud in camera-0 coordinates
@@ -741,11 +752,16 @@ SYNOPSIS
 
 As shown in the example above, we can perform stereo processing by building
 rectified models and transformation maps, rectifying our images, and then doing
-stereo matching to get pixel disparities. This function performs the last step:
-converting pixel disparities to ranges.
+stereo matching to get pixel disparities. The disparities can be converted to
+usable geometry by calling one of two functions:
 
-In the most common usage we take a full disparity IMAGE, and then convert it to
-a range IMAGE. In this common case we call
+- stereo_range() to convert pixel disparities to ranges
+
+- stereo_unproject() to convert pixel disparities to a point cloud. This is a
+  superset of stereo_range()
+
+In the most common usage of stereo_range() we take a full disparity IMAGE, and
+then convert it to a range IMAGE. In this common case we call
 
     range_image = mrcal.stereo_range(disparity_image, models_rectified)
 
@@ -889,8 +905,21 @@ RETURNED VALUES
 
     _validate_models_rectified(models_rectified)
 
+    # I want to support scalar disparities. If one is given, I convert it into
+    # an array of shape (1,), and then pull it out at the end
+    is_scalar = False
+    try:
+        s = disparity.shape
+    except:
+        is_scalar = True
+    if not is_scalar:
+        if len(s) == 0:
+            is_scalar = True
+    if is_scalar:
+        disparity = np.array((disparity,),)
+
     W,H = models_rectified[0].imagersize()
-    if qrect0 is None and disparity.shape != (H,W):
+    if qrect0 is None and np.any(disparity.shape - np.array((H,W),dtype=int)):
         raise Exception(f"qrect0 is None, so the disparity image must have the full dimensions of a rectified image")
 
     intrinsics = models_rectified[0].intrinsics()
@@ -944,13 +973,175 @@ RETURNED VALUES
             ((s_sq_recip + tanaz0*tanaz1) / tanaz0_tanaz1 + \
              tanaz0)
 
+        mask_invalid += ~np.isfinite(r)
+
     r[mask_invalid] = 0
 
+    if is_scalar:
+        r = r[0]
     return r
 
 
+def stereo_unproject(disparity,
+                     models_rectified,
+                     *,
+                     ranges          = None,
+                     disparity_scale = 1,
+                     qrect0          = None):
+
+    r'''Compute a point cloud from observed disparities
+
+SYNOPSIS
+
+    import sys
+    import mrcal
+    import cv2
+    import numpy as np
+    import numpysane as nps
+
+    models = [ mrcal.cameramodel(f) \
+               for f in ('left.cameramodel',
+                         'right.cameramodel') ]
+
+    images = [ mrcal.load_image(f) \
+               for f in ('left.jpg', 'right.jpg') ]
+
+    models_rectified = \
+        mrcal.rectified_system(models,
+                               az_fov_deg = 120,
+                               el_fov_deg = 100)
+
+    rectification_maps = mrcal.rectification_maps(models, models_rectified)
+
+    images_rectified = [ mrcal.transform_image(images[i], rectification_maps[i]) \
+                         for i in range(2) ]
+
+    # Find stereo correspondences using OpenCV
+    block_size = 3
+    max_disp   = 160 # in pixels
+    matcher = \
+        cv2.StereoSGBM_create(minDisparity      = 0,
+                              numDisparities    = max_disp,
+                              blockSize         = block_size,
+                              P1                = 8 *3*block_size*block_size,
+                              P2                = 32*3*block_size*block_size,
+                              uniquenessRatio   = 5,
+
+                              disp12MaxDiff     = 1,
+                              speckleWindowSize = 50,
+                              speckleRange      = 1)
+    disparity16 = matcher.compute(*images_rectified) # in pixels*16
+
+    # Point cloud in rectified camera-0 coordinates
+    # shape (H,W,3)
+    p_rect0 = mrcal.stereo_unproject( disparity16,
+                                      models_rectified,
+                                      disparity_scale = 16 )
+
+    Rt_cam0_rect0 = mrcal.compose_Rt( models          [0].extrinsics_Rt_fromref(),
+                                      models_rectified[0].extrinsics_Rt_toref() )
+
+    # Point cloud in camera-0 coordinates
+    # shape (H,W,3)
+    p_cam0 = mrcal.transform_point_Rt(Rt_cam0_rect0, p_rect0)
+
+As shown in the example above, we can perform stereo processing by building
+rectified models and transformation maps, rectifying our images, and then doing
+stereo matching to get pixel disparities. The disparities can be converted to
+usable geometry by calling one of two functions:
+
+- stereo_range() to convert pixel disparities to ranges
+
+- stereo_unproject() to convert pixel disparities to a point cloud, each point in
+  the rectified-camera-0 coordinates. This is a superset of stereo_range()
+
+In the most common usage of stereo_unproject() we take a full disparity IMAGE,
+and then convert it to a dense point cloud: one point per pixel. In this common
+case we call
+
+    p_rect0 = mrcal.stereo_unproject(disparity_image, models_rectified)
+
+If we aren't processing the full disparity image, we can pass in an array of
+rectified pixel coordinates (in the first rectified camera) in the "qrect0"
+argument. These must be broadcastable with the disparity argument.
+
+The arguments are identical to those accepted by stereo_range(), except an
+optional "ranges" argument is accepted. This can be given in lieu of the
+"disparity" argument, if we already have ranges returned by stereo_range().
+Exactly one of (disparity,ranges) must be given as non-None. "ranges" is a
+keyword-only argument, while "disparity" is positional. So if passing ranges,
+the disparity must be explicitly given as None:
+
+    p_rect0 = mrcal.stereo_unproject( disparity        = None,
+                                      models_rectified = models_rectified,
+                                      ranges           = ranges )
+
+ARGUMENTS
+
+- disparity: a numpy array of disparities being processed. If disparity_scale is
+  omitted, this array contains floating-point disparity values in PIXELS. Many
+  stereo-matching algorithms produce integer disparities, in units of some
+  constant number of pixels (the OpenCV StereoSGBM and StereoBM routines use
+  16). In this common case, you can pass the integer scaled disparities here,
+  with the scale factor in disparity_scale. Any array shape is supported. In the
+  common case of a disparity IMAGE, this is an array of shape (Nel, Naz)
+
+- models_rectified: the pair of rectified models, corresponding to the input
+  images. Usually this is returned by mrcal.rectified_system()
+
+- ranges: optional numpy array with the ranges returned by stereo_range().
+  Exactly one of (disparity,ranges) should be given as non-None.
+
+- disparity_scale: optional scale factor for the "disparity" array. If omitted,
+  the "disparity" array is assumed to contain the disparities, in pixels.
+  Otherwise it contains data in the units of 1/disparity_scale pixels.
+
+- qrect0: optional array of rectified camera0 pixel coordinates corresponding to
+  the given disparities. By default, a full disparity image is assumed.
+  Otherwise we use the given rectified coordinates. The shape of this array must
+  be broadcasting-compatible with the disparity array. See the
+  description above.
+
+RETURNED VALUES
+
+- An array of points of the same dimensionality as the input disparity (or
+  ranges) array. Contains floating-point data. Invalid or missing points are
+  represented as (0,0,0).
+
+    '''
+
+    if (ranges is     None and disparity is     None) or \
+       (ranges is not None and disparity is not None):
+        raise Exception("Exactly one of (disparity,ranges) must be non-None")
+
+    if ranges is None:
+        ranges = stereo_range(disparity,
+                              models_rectified,
+                              disparity_scale = disparity_scale,
+                              qrect0          = qrect0)
+
+    if qrect0 is None:
+
+        W,H = models_rectified[0].imagersize()
+
+        # shape (H,W,2)
+        qrect0 = np.ascontiguousarray( \
+               nps.mv( nps.cat( *np.meshgrid(np.arange(W,dtype=float),
+                                             np.arange(H,dtype=float))),
+                       0, -1))
+
+    # shape (..., 3)
+    vrect0 = mrcal.unproject(qrect0, *models_rectified[0].intrinsics(),
+                             normalize = True)
+    # shape (..., 3)
+    p_rect0 = vrect0 * nps.dummy(ranges, axis = -1)
+
+    return p_rect0
+
+
 def match_feature( image0, image1,
                    q0,
+                   *,
                    search_radius1,
                    template_size1,
                    q1_estimate      = None,
