@@ -14,7 +14,7 @@ import numpysane as nps
 import sys
 import mrcal
 
-def ref_calibration_object(W, H, object_spacing, calobject_warp=None):
+def ref_calibration_object(W, H, object_spacing, *, calobject_warp=None):
     r'''Return the geometry of the calibration object
 
 SYNOPSIS
@@ -89,25 +89,44 @@ ARGUMENTS
 - H: how many points we have in the vertical direction
 
 - object_spacing: the distance between adjacent points in the calibration
-  object. A square object is assumed, so the vertical and horizontal distances
-  are assumed to be identical.
+  object. If a scalar is given, a square object is assumed, and the vertical and
+  horizontal distances are assumed to be identical. An array of shape (..., 2)
+  can be given: the last dimension is (spacing_h, spacing_w), and the preceding
+  dimensions are used for broadcasting
 
 - calobject_warp: optional array of shape (2,) defaults to None. Describes the
   warping of the calibration object. If None, the object is flat. If an array is
   given, the values describe the maximum additive deflection along the x and y
-  axes
+  axes. Extended array can be given for broadcasting
+
+This function supports broadcasting across object_spacing and calobject_warp
 
 RETURNED VALUES
 
-The calibration object geometry in a (H,W,3) array
+The calibration object geometry in a (..., H,W,3) array, with the leading
+dimensions set by the broadcasting rules
 
     '''
 
-    xx,yy       = np.meshgrid( np.arange(W,dtype=float), np.arange(H,dtype=float))
+    # shape (H,W)
+    xx,yy = np.meshgrid( np.arange(W,dtype=float), np.arange(H,dtype=float))
+
+    # shape (H,W,3)
     full_object = nps.glue(nps.mv( nps.cat(xx,yy), 0, -1),
                            np.zeros((H,W,1)),
-                           axis=-1) # shape (H,W,3)
-    full_object *= object_spacing
+                           axis=-1)
+
+    # object_spacing has shape (..., 2)
+    object_spacing = np.array(object_spacing)
+    if object_spacing.ndim == 0:
+        object_spacing = np.array((1,1))*object_spacing
+    object_spacing = nps.dummy(object_spacing, -2,-2)
+    # object_spacing now has shape (..., 1,1,2)
+
+    if object_spacing.ndim > 3:
+        # extend full_object to the output shape I want
+        full_object = full_object * np.ones( object_spacing.shape[:-3] + (1,1,1) )
+    full_object[..., :2] *= object_spacing
 
     if calobject_warp is not None:
         xr = xx / (W-1)
@@ -115,15 +134,24 @@ The calibration object geometry in a (H,W,3) array
         dx = 4. * xr * (1. - xr)
         dy = 4. * yr * (1. - yr)
 
-        full_object[..., 2] += calobject_warp[0] * dx
-        full_object[..., 2] += calobject_warp[1] * dy
+        # To allow broadcasting over calobject_warp
+        if calobject_warp.ndim > 1:
+            # shape (..., 1,1,2)
+            calobject_warp = nps.dummy(calobject_warp, -2,-2)
+            # extend full_object to the output shape I want
+            full_object = full_object * np.ones( calobject_warp.shape[:-3] + (1,1,1) )
+        full_object[..., 2] += calobject_warp[...,0] * dx
+        full_object[..., 2] += calobject_warp[...,1] * dy
 
     return full_object
 
 
 def synthesize_board_observations(models,
-                                  object_width_n,object_height_n,
-                                  object_spacing, calobject_warp,
+                                  *,
+                                  object_width_n,
+                                  object_height_n,
+                                  object_spacing,
+                                  calobject_warp,
                                   rt_ref_boardcenter,
                                   rt_ref_boardcenter__noiseradius,
                                   Nframes,
@@ -143,14 +171,17 @@ SYNOPSIS
           models,
 
           # board geometry
-          10,12,0.1,None,
+          object_width_n  = 10,
+          object_height_n = 12,
+          object_spacing  = 0.1,
+          calobject_warp  = None,
 
           # mean board pose and the radius of the added uniform noise
-          rt_ref_boardcenter,
-          rt_ref_boardcenter__noiseradius,
+          rt_ref_boardcenter              = rt_ref_boardcenter
+          rt_ref_boardcenter__noiseradius = rt_ref_boardcenter__noiseradius,
 
           # How many frames we want
-          100,
+          Nframes = 100
 
           which = 'some-cameras-must-see-half-board')
 
@@ -261,10 +292,13 @@ We return a tuple:
                                imagersize = np.array((2000,2000)) )
     Rt_ref_boardref = \
         mrcal.synthesize_board_observations([model],
-                                            5,20,0.1,None,
-                                            nps.glue(r, np.array((0,0,3.)), axis=-1),
-                                            np.array((0,0,0., 0,0,0)),
-                                            1) [1]
+                                            object_width_n                  = 5,
+                                            object_height_n                 = 20,
+                                            object_spacing                  = 0.1,
+                                            calobject_warp                  = None,
+                                            rt_ref_boardcenter              = nps.glue(r, np.array((0,0,3.)), axis=-1),
+                                            rt_ref_boardcenter__noiseradius = np.array((0,0,0., 0,0,0)),
+                                            Nframes                         = 1) [1]
     mrcal.show_geometry( models_or_extrinsics_rt_fromref = np.zeros((1,1,6), dtype=float),
                          frames_rt_toref                 = mrcal.rt_from_Rt(Rt_ref_boardref),
                          object_width_n                  = 20,
@@ -298,7 +332,8 @@ We return a tuple:
     # shape: (Nh,Nw,3)
     board_reference = \
         mrcal.ref_calibration_object(object_width_n,object_height_n,
-                                     object_spacing,calobject_warp) - \
+                                     object_spacing,
+                                     calobject_warp = calobject_warp) - \
         board_center
 
     # Transformation from the board returned by ref_calibration_object() to
