@@ -18,8 +18,8 @@ int main(int argc, char* argv[] )
         "\n"
         "The lensmodels are given as the expected strings. Splined stereographic models\n"
         "MUST be given as either of\n"
-        "  MRCAL_LENSMODEL_SPLINED_STEREOGRAPHIC_2\n"
-        "  MRCAL_LENSMODEL_SPLINED_STEREOGRAPHIC_3\n"
+        "  LENSMODEL_SPLINED_STEREOGRAPHIC_2\n"
+        "  LENSMODEL_SPLINED_STEREOGRAPHIC_3\n"
         "\n"
         "problem-selections are a list of parameters we're optimizing. This is some set of\n"
         "  intrinsic-core\n"
