@@ -109,8 +109,17 @@ def parse_args():
     parser.add_argument('--extra-observation-at',
                         type=float,
                         help='''Adds one extra observation at the given distance''')
+    parser.add_argument('--range-to-boards',
+                        type=float,
+                        default=4.0,
+                        help='''Nominal range to the simulated chessboards''')
     parser.add_argument('--reproject-perturbed',
-                        choices=('mean-frames', 'mean-frames-using-meanq', 'mean-frames-using-meanq-penalize-big-shifts', 'fit-boards-ref', 'diff'),
+                        choices=('mean-frames',
+                                 'mean-frames-using-meanq',
+                                 'mean-frames-using-meanq-penalize-big-shifts',
+                                 'fit-boards-ref',
+                                 'diff',
+                                 'optimize-cross-reprojection-error'),
                         default = 'mean-frames',
                         help='''Which reproject-after-perturbation method to use. This is for experiments.
                         Some of these methods will be probably wrong.''')
@@ -241,7 +250,8 @@ args.Nframes =                                         \
                          extrinsics_rt_fromref_true,
                          calobject_warp_true,
                          fixedframes,
-                         testdir)
+                         testdir,
+                         range_to_boards = args.range_to_boards)
 
 # I evaluate the projection uncertainty of this vector. In each camera. I'd like
 # it to be center-ish, but not AT the center. So I look at 1/3 (w,h). I want
@@ -342,6 +352,8 @@ def reproject_perturbed__mean_frames(q, distance,
                                      baseline_rt_ref_frame,
                                      # shape (2)
                                      baseline_calobject_warp,
+                                     # dict
+                                     baseline_optimization_inputs,
 
                                      # shape (..., Ncameras, Nintrinsics)
                                      query_intrinsics,
@@ -350,7 +362,9 @@ def reproject_perturbed__mean_frames(q, distance,
                                      # shape (..., Nframes, 6)
                                      query_rt_ref_frame,
                                      # shape (..., 2)
-                                     query_calobject_warp):
+                                     query_calobject_warp,
+                                     # list of dicts of length ...
+                                     query_optimization_inputs):
     r'''Reproject by computing the mean in the space of frames
 
 This is what the uncertainty computation does (as of 2020/10/26). The implied
@@ -436,6 +450,8 @@ def reproject_perturbed__fit_boards_ref(q, distance,
                                         baseline_rt_ref_frame,
                                         # shape (2)
                                         baseline_calobject_warp,
+                                        # dict
+                                        baseline_optimization_inputs,
 
                                         # shape (..., Ncameras, Nintrinsics)
                                         query_intrinsics,
@@ -444,26 +460,29 @@ def reproject_perturbed__fit_boards_ref(q, distance,
                                         # shape (..., Nframes, 6)
                                         query_rt_ref_frame,
                                         # shape (..., 2)
-                                        query_calobject_warp):
+                                        query_calobject_warp,
+                                        # list of dicts of length ...
+                                        query_optimization_inputs):
+
     r'''Reproject by explicitly computing a procrustes fit to align the reference
     coordinate systems of the two solves. We match up the two sets of chessboard
     points
 
     '''
 
-    calobject_height,calobject_width = optimization_inputs_baseline['observations_board'].shape[1:3]
+    calobject_height,calobject_width = baseline_optimization_inputs['observations_board'].shape[1:3]
 
     # shape (Nsamples, Nh, Nw, 3)
     if query_calobject_warp.ndim > 1:
         calibration_object_query = \
             nps.cat(*[ mrcal.ref_calibration_object(calobject_width, calobject_height,
-                                                    optimization_inputs_baseline['calibration_object_spacing'],
+                                                    baseline_optimization_inputs['calibration_object_spacing'],
                                                     calobject_warp=calobject_warp) \
                        for calobject_warp in query_calobject_warp] )
     else:
         calibration_object_query = \
             mrcal.ref_calibration_object(calobject_width, calobject_height,
-                                         optimization_inputs_baseline['calibration_object_spacing'],
+                                         baseline_optimization_inputs['calibration_object_spacing'],
                                          calobject_warp=query_calobject_warp)
 
     # shape (Nsamples, Nframes, Nh, Nw, 3)
@@ -475,7 +494,7 @@ def reproject_perturbed__fit_boards_ref(q, distance,
     # shape (Nh, Nw, 3)
     calibration_object_baseline = \
         mrcal.ref_calibration_object(calobject_width, calobject_height,
-                                     optimization_inputs_baseline['calibration_object_spacing'],
+                                     baseline_optimization_inputs['calibration_object_spacing'],
                                      calobject_warp=baseline_calobject_warp)
     # frames_ref.shape is (Nframes, 6)
 
@@ -523,14 +542,19 @@ def reproject_perturbed__fit_boards_ref(q, distance,
 # The others broadcast implicitly, while THIS main function really cannot handle
 # outer dimensions, and needs an explicit broadcasting loop
 @nps.broadcast_define(((2,), (),
+
                        ('Ncameras', 'Nintrinsics'),
                        ('Ncameras', 6),
                        ('Nframes', 6),
                        (2,),
+                       (),
+
                        ('Ncameras', 'Nintrinsics'),
                        ('Ncameras', 6),
                        ('Nframes', 6),
-                       (2,),),
+                       (2,),
+                       ()),
+
                       ('Ncameras',2))
 def reproject_perturbed__diff(q, distance,
                               # shape (Ncameras, Nintrinsics)
@@ -541,6 +565,8 @@ def reproject_perturbed__diff(q, distance,
                               baseline_rt_ref_frame,
                               # shape (2)
                               baseline_calobject_warp,
+                              # dict
+                              baseline_optimization_inputs,
 
                               # shape (Ncameras, Nintrinsics)
                               query_intrinsics,
@@ -549,7 +575,10 @@ def reproject_perturbed__diff(q, distance,
                               # shape (Nframes, 6)
                               query_rt_ref_frame,
                               # shape (2)
-                              query_calobject_warp):
+                              query_calobject_warp,
+                              # dict
+                              query_optimization_inputs):
+
     r'''Reproject by using the "diff" method to compute a rotation
 
     '''
@@ -584,6 +613,434 @@ def reproject_perturbed__diff(q, distance,
                        lensmodel, query_intrinsics)
 
 
+def reproject_perturbed__optimize_cross_reprojection_error(q, distance,
+
+                                                           # shape (Ncameras, Nintrinsics)
+                                                           baseline_intrinsics,
+                                                           # shape (Ncameras, 6)
+                                                           baseline_rt_cam_ref,
+                                                           # shape (Nframes, 6)
+                                                           baseline_rt_ref_frame,
+                                                           # shape (2)
+                                                           baseline_calobject_warp,
+                                                           # dict
+                                                           baseline_optimization_inputs,
+
+                                                           # shape (..., Ncameras, Nintrinsics)
+                                                           query_intrinsics,
+                                                           # shape (..., Ncameras, 6)
+                                                           query_rt_cam_ref,
+                                                           # shape (..., Nframes, 6)
+                                                           query_rt_ref_frame,
+                                                           # shape (..., 2)
+                                                           query_calobject_warp,
+                                                           # list of dicts of length ...
+                                                           query_optimization_inputs):
+
+    r'''Reproject by explicitly computing a ref-refperturbed transformation
+
+I have a baseline solve (parameter vector b) and a perturbed solve (parameter
+vector bperturbed) obtained from perturbing the observations qref and
+re-optimizing. I also have an arbitrary baseline query pixel q and distance d
+from which I compute the perturbed reprojection qperturbed.
+
+I need to eventually compute Var(qperturbed). I linearize everything to get
+delta_qperturbed ~ dqperturbed/dbperturbed dbperturbed/dqref delta_qref. Let
+
+  L = dqperturbed/dbperturbed
+  M = dbperturbed/dqref
+
+so
+
+  delta_qperturbed = L M delta_qref
+
+Then
+
+  Var(qperturbed) = L M Var(qref) Mt Lt.
+
+I have M from the usual uncertainty propagation logic, so I just need L =
+dqperturbed/dbperturbed
+
+In my usual least squares solve each chessboard point produces two elements of
+the measurements x:
+
+  x_point =
+    qref - project(intrinsics,
+                   T_cam_ref T_ref_frame p)
+
+Here I optimize the reprojection error looking at the PERTURBED
+chessboard,frames,points and the UNPERTURBED camera intrinsics, extrinsics. This
+requires computing a ref transformation to take into account the shifting
+reference frame that results when re-optimizing:
+
+  x_cross_point =
+    qref - project(intrinsics,
+                   T_cam_ref T_ref_refperturbed T_refperturbed_frameperturbed p_perturbed)
+
+And I reoptimize norm2(x_cross_point) by varying T_ref_refperturbed. This is
+parametrized as rt_ref_refperturbed. Let J_cross =
+dx_cross_point/drt_ref_refperturbed. I assume everything is locally linear, as
+defined by J_cross, and I take a single Newton step. I minimize
+
+  E = norm2(x_cross_point0 + dx_cross_point)
+
+I set the derivative to 0:
+
+  0 = dE/drt_ref_refperturbed ~ (x_cross_point0 + dx_cross_point)t J_cross
+
+-> J_cross_t x_cross_point0 = -J_cross_t dx_cross_point
+
+Furthermore, dx_cross_point = J_cross drt_ref_refperturbed, so
+
+  J_cross_t x_cross_point0 = -J_cross_t J_cross drt_ref_refperturbed
+
+and
+
+  drt_ref_refperturbed = -inv(J_cross_t J_cross) J_cross_t x_cross_point0
+
+Everything I'm looking at implies small deviations, so I use
+T_ref_refperturbed=identity (rt_ref_refperturbed = 0) as the operating point,
+and
+
+  rt_ref_refperturbed = -inv(J_cross_t J_cross) J_cross_t x_cross_point0
+
+This is good, but implies that J_cross needs to be computed directly by
+propagating gradients from the projection and the transform composition. We can
+do better.
+
+Since everything I'm looking at is near the original solution to the main
+optimization problem, I can look at EVERYTHING in the linear space defined by
+the optimal measurements x and their gradient J. The x_cross_point expression
+can be simplified:
+
+  x_cross_point =
+    x_cross_point0 +
+    J_intrinsics     dintrinsics +
+    J_extrinsics     drt_cam_ref +
+    J_frame          drt_ref_frame +
+    J_calobject_warp dcalobject_warp
+
+In the expression above we use the unperturbed intrinsics and extrinsics, so
+dintrinsics = 0 and drt_cam_ref = 0. The shift in the calibration object warp
+comes directly from the shift in parameters: dcalobject_warp = M[calobject_warp]
+delta_qref. That leaves drt_ref_frame. This represents a shift from the
+optimized rt_ref_frame to rt_ref_frameperturbed =
+compose_rt(rt_ref_refperturbed, rt_refperturbed_frameperturbed). For
+x_cross_point0, I have rt_ref_refperturbed = 0, so there I have drt_ref_frame =
+M[frame] delta_qref. And for the gradient I have:
+
+  J_cross = dx_cross_point/drt_ref_refperturbed
+          = J_frame drt_ref_frame/drt_ref_refperturbed
+
+which I can obtain from the transform composition functions.
+
+Now that I have rt_ref_refperturbed, I can use it to compute qperturbed. This
+can accept arbitrary q, not just those in the solve, so I actually need to
+compute projections, rather than looking at a linearized space defined by J
+
+I have
+
+  pcam = unproject(intrinsics, q)
+  pcam_perturbed = T_camperturbed_refperturbed T_refperturbed_ref T_ref_cam pcam
+  qperturbed = project(intrinsics_perturbed, pcam_perturbed)
+
+  dqperturbed/dbperturbed[intrinsics] comes directly from this project()
+
+  dqperturbed/dbperturbed[extrinsics] = dqperturbed/dpcam_perturbed
+                                        (dpcam_perturbed/dextrinsics + dpcam_perturbed/drt_ref_refperturbed drt_ref_refperturbed/dextrinsics)
+
+
+So I need gradients of rt_ref_refperturbed in respect to p_perturbed
+
+    '''
+
+    if fixedframes:
+        raise Exception("reproject_perturbed__optimize_cross_reprojection_error(fixedframes = True) is not yet implemented")
+
+    def compose_rt3_withgrad_drt1(rt0, rt1, rt2):
+        rt01,drt01_drt0,drt01_drt1 = \
+            mrcal.compose_rt(rt0, rt1, get_gradients = True)
+        rt012,drt012_drt01,drt012_drt2 = \
+            mrcal.compose_rt(rt01, rt2, get_gradients = True)
+        drt012_drt1 = nps.matmult(drt012_drt01, drt01_drt1)
+        return rt012,drt012_drt1
+
+    def transform_point_rt3_withgrad_drt1(rt0, rt1, rt2, p):
+        rt012,drt012_drt1 = compose_rt3_withgrad_drt1(rt0,rt1,rt2)
+        pp, dpp_drt012, dpp_dp = mrcal.transform_point_rt(rt012, p, get_gradients = True)
+        dpp_drt1 = nps.matmult(dpp_drt012, drt012_drt1)
+        return pp, dpp_drt1
+
+    def transform_point_identity_gradient(p):
+        r'''Computes dprot/drt where prot = transform(rt,p) and rt = identity
+
+prot = rotate(p) + t, so clearly dprot/dt = I
+
+Now let's look at the rotation
+
+mrcal_transform_point_rt_full() from rotate_point_r_core() defines
+prot = rotate(rt, p) at rt=identity:
+
+  const val_withgrad_t<N> cross[3] =
+      {
+          (rg[1]*x_ing[2] - rg[2]*x_ing[1]),
+          (rg[2]*x_ing[0] - rg[0]*x_ing[2]),
+          (rg[0]*x_ing[1] - rg[1]*x_ing[0])
+      };
+  const val_withgrad_t<N> inner =
+      rg[0]*x_ing[0] +
+      rg[1]*x_ing[1] +
+      rg[2]*x_ing[2];
+
+  // Small rotation. I don't want to divide by 0, so I take the limit
+  //   lim(th->0, xrot) =
+  //     = x + cross(r, x) + r rt x lim(th->0, (1 - cos(th)) / (th*th))
+  //     = x + cross(r, x) + r rt x lim(th->0, sin(th) / (2*th))
+  //     = x + cross(r, x) + r rt x/2
+  for(int i=0; i<3; i++)
+      x_outg[i] =
+          x_ing[i] +
+          cross[i] +
+          rg[i]*inner / 2.;
+
+So dprot/dr = dcross/dr + d(r*inner / 2)/dr =
+              [  0  p2 -p1]
+            = [-p2   0  p0] + (inner I + outer(r,p))/2
+              [ p1 -p0   0]
+
+At r=identity I have r = 0, so
+
+              [  0  p2 -p1]
+  dprot/dr  = [-p2   0  p0]
+              [ p1 -p0   0]
+
+This is the usual skew-symmetric matrix that appears in the matrix form of the
+cross product
+
+        '''
+
+        # strange-looking implementation to make broadcasting work
+        dprot_drt = np.zeros(p.shape[:-1] + (3,6), dtype=float)
+
+        dprot_drt[...,0,1] =  p[...,2]
+        dprot_drt[...,0,2] = -p[...,1]
+        dprot_drt[...,1,0] = -p[...,2]
+        dprot_drt[...,1,2] =  p[...,0]
+        dprot_drt[...,2,0] =  p[...,1]
+        dprot_drt[...,2,1] = -p[...,0]
+
+        dprot_drt[...,0,0+3] = 1.
+        dprot_drt[...,1,1+3] = 1.
+        dprot_drt[...,2,2+3] = 1.
+
+        _,dprot_drt_reference,_ = \
+            mrcal.transform_point_rt(mrcal.identity_rt(), p,
+                                     get_gradients=True)
+        if nps.norm2((dprot_drt-dprot_drt_reference).ravel()) > 1e-10:
+            raise Exception("transform_point_identity_gradient() is computing the wrong thing. This is a bug")
+
+        return dprot_drt
+
+
+
+
+
+
+
+    observations_board = \
+        baseline_optimization_inputs.get('observations_board')
+    indices_frame_camintrinsics_camextrinsics = \
+        baseline_optimization_inputs['indices_frame_camintrinsics_camextrinsics']
+
+    object_width_n      = observations_board.shape[-2]
+    object_height_n     = observations_board.shape[-3]
+    object_spacing      = baseline_optimization_inputs['calibration_object_spacing']
+    # shape (Nh,Nw,3)
+    calibration_object_baseline = \
+        mrcal.ref_calibration_object(object_width_n,
+                                     object_height_n,
+                                     object_spacing,
+                                     calobject_warp = baseline_calobject_warp)
+
+    # need to define the broadcasted function myself
+    @nps.broadcast_define( ((2,),) )
+    def ref_calibration_object(calobject_warp):
+        return \
+            mrcal.ref_calibration_object(object_width_n,
+                                         object_height_n,
+                                         object_spacing,
+                                         calobject_warp = calobject_warp)
+    calibration_object_query = \
+        ref_calibration_object(query_calobject_warp)
+
+    weight = observations_board[...,2]
+
+    # shape (Nobservations, 6)
+    rt_ref_frame_all = \
+        baseline_rt_ref_frame[ ..., indices_frame_camintrinsics_camextrinsics[:,0], :]
+    # shape (..., Nobservations, 6)
+    rt_refperturbed_frameperturbed_all = \
+        query_rt_ref_frame[ ..., indices_frame_camintrinsics_camextrinsics[:,0], :]
+    # shape (Nobservations, Nintrinsics)
+    intrinsics_all = \
+        baseline_intrinsics[ indices_frame_camintrinsics_camextrinsics[:,1], :]
+    if nps.norm2(baseline_rt_cam_ref[0]) > 1e-12:
+        raise Exception("I'm assuming a vanilla calibration problem reference at cam0")
+    # shape (Nobservations, 6)
+    rt_cam_ref_all = \
+        baseline_rt_cam_ref[ indices_frame_camintrinsics_camextrinsics[:,2]+1, :]
+
+    # I look at the un-perturbed data first, to double-check that I'm doing the
+    # right thing. This is purely a self-checking step. I don't need to do it
+    if 1:
+
+        if 1:
+            # shape (Nobservations,Nh,Nw,3),
+            pref = mrcal.transform_point_rt(nps.dummy(rt_ref_frame_all, -2,-2),
+                                            calibration_object_baseline)
+            pcam = mrcal.transform_point_rt(nps.dummy(rt_cam_ref_all, -2,-2),
+                                            pref)
+        else:
+            # More complex form, using custom function.
+
+            # shape (..., Nobservations,Nh,Nw,3),
+            pcam, _ = \
+                transform_point_rt3_withgrad_drt1(nps.dummy(rt_cam_ref_all, -2,-2),
+                                                  mrcal.identity_rt(),
+                                                  p_ref)
+
+        # shape (..., Nobservations,Nh,Nw,2)
+        qq = \
+            mrcal.project(pcam,
+                          baseline_optimization_inputs['lensmodel'],
+                          nps.dummy(intrinsics_all, -2,-2))
+        x = (qq - observations_board[...,:2])*nps.dummy(weight,-1)
+        x[...,weight<=0,:] = 0 # outliers
+        x = x.ravel()
+        if len(x) != mrcal.num_measurements_boards(**baseline_optimization_inputs):
+            raise Exception("Unexpected len(x). This is a bug")
+        x_baseline = mrcal.optimizer_callback(**baseline_optimization_inputs)[1]
+        if nps.norm2(x - x_baseline[:len(x)]) > 1e-12:
+            raise Exception("Unexpected x. This is a bug")
+
+        E_baseline = nps.norm2(x)
+
+    # Alright. I'm mimicking the optimization function well-enough. Let's look
+    # at the crossed quantities
+
+    if 1:
+
+        # shape (..., Nobservations,Nh,Nw,3),
+        pref = mrcal.transform_point_rt( nps.dummy(rt_refperturbed_frameperturbed_all, -2,-2),
+                                         nps.mv(calibration_object_query,-4,-5))
+        dpref_drt_ref_refperturbed = transform_point_identity_gradient(pref)
+        # shape (..., Nobservations,Nh,Nw,3),
+        #       (..., Nobservations,Nh,Nw,3,6)
+        pcam, _, dpcam_dpref = \
+            mrcal.transform_point_rt(nps.dummy(rt_cam_ref_all, -2,-2),
+                                     pref,
+                                     get_gradients = True)
+        dpcam_drt_ref_refperturbed = nps.matmult(dpcam_dpref, dpref_drt_ref_refperturbed)
+
+    else:
+        # More complex form, using custom function.
+
+        # shape (..., Nobservations,Nh,Nw,3),
+        #       (..., Nobservations,Nh,Nw,3,6)
+        pcam, dpcam_drt_ref_refperturbed = \
+            transform_point_rt3_withgrad_drt1(nps.dummy(rt_cam_ref_all, -2,-2),
+                                              mrcal.identity_rt(),
+                                              nps.dummy(rt_refperturbed_frameperturbed_all, -2,-2),
+                                              nps.mv(calibration_object_query,-4,-5))
+
+    # shape (..., Nobservations,Nh,Nw,2)
+    #       (..., Nobservations,Nh,Nw,2,3)
+    qq,dq_dpcam,_ = \
+        mrcal.project(pcam,
+                      baseline_optimization_inputs['lensmodel'],
+                      nps.dummy(intrinsics_all, -2,-2),
+                      get_gradients = True)
+    x = (qq - observations_board[...,:2])*nps.dummy(weight,-1)
+    x[...,weight<=0,:] = 0 # outliers
+    # shape (..., Nobservations*Nh*Nw*2)
+    x = nps.clump(x, n=-4)
+
+    E_perturbed = nps.norm2(x)
+
+    dx_dpcam = dq_dpcam*nps.dummy(weight,-1,-1)
+    dx_dpcam[...,weight<=0,:,:] = 0 # outliers
+    dx_drt_ref_refperturbed = nps.matmult(dx_dpcam, dpcam_drt_ref_refperturbed)
+    # shape (...,Nobservations,Nh,Nw,2,6) ->
+    #       (...,Nobservations*Nh*Nw*2,6) ->
+    dx_drt_ref_refperturbed = \
+        nps.mv(nps.clump(nps.mv(dx_drt_ref_refperturbed, -1, -5),
+                         n = -4),
+               -2, -1)
+    J = dx_drt_ref_refperturbed
+
+    # need to define the broadcasted function myself
+    @nps.broadcast_define((('N',6),('N',)),
+                          (6,))
+    def lstsq(J,x):
+        # -inv(JtJ)Jt x0
+        return np.linalg.lstsq(J, x, rcond = None)[0]
+    rt_ref_refperturbed = -lstsq(J, x)
+
+    # I have a 1-step solve. Let's look at the error to confirm that it's
+    # smaller
+    if 1:
+        # shape (..., Nobservations,Nh,Nw,3)
+        pcam = \
+            mrcal.transform_point_rt( mrcal.compose_rt(nps.dummy(rt_cam_ref_all, -2,-2),
+                                                       nps.mv(rt_ref_refperturbed, -2,-5),
+                                                       nps.dummy(rt_refperturbed_frameperturbed_all, -2,-2)),
+                                      nps.mv(calibration_object_query,-4,-5))
+
+        # shape (..., Nobservations,Nh,Nw,2)
+        qq = \
+            mrcal.project(pcam,
+                          baseline_optimization_inputs['lensmodel'],
+                          nps.dummy(intrinsics_all, -2,-2),
+                          get_gradients = False)
+        x = (qq - observations_board[...,:2])*nps.dummy(weight,-1)
+        x[...,weight<=0,:] = 0 # outliers
+        # shape (..., Nobservations*Nh*Nw*2)
+        x = nps.clump(x, n=-4)
+
+        E_perturbed_solvedref = nps.norm2(x)
+
+        print(f"RMS error baseline            = {np.sqrt(E_baseline            / (x.shape[-1]/2))} pixels")
+        print(f"RMS error perturbed           = {np.sqrt(E_perturbed           / (x.shape[-1]/2))} pixels")
+        print(f"RMS error perturbed_solvedref = {np.sqrt(E_perturbed_solvedref / (x.shape[-1]/2))} pixels")
+
+
+    # shape (Ncameras, 3)
+    p_cam_baseline = mrcal.unproject(q, lensmodel, baseline_intrinsics,
+                                     normalize = True) * distance
+    # shape (Ncameras, 3)
+    p_ref_baseline = \
+        mrcal.transform_point_rt( mrcal.invert_rt(baseline_rt_cam_ref),
+                                  p_cam_baseline )
+    # shape (...,Ncameras, 3)
+    p_ref_query = \
+        mrcal.transform_point_rt( nps.dummy(rt_ref_refperturbed, -2),
+                                  p_ref_baseline,
+                                  inverted = True )
+
+    # shape (..., Ncameras, 3)
+    p_cam_query = \
+        mrcal.transform_point_rt(query_rt_cam_ref, p_ref_query)
+
+    # shape (..., Ncameras, 2)
+    #
+    # If ... was (), the current code sets it to (1,). So I force the right
+    # shape
+    return mrcal.project(p_cam_query, lensmodel, query_intrinsics). \
+        reshape( *query_intrinsics.shape[:-1],
+                 2)
+
+
+
 # Which implementation we're using. Use the method that matches the uncertainty
 # computation. Thus the sampled ellipsoids should match the ellipsoids reported
 # by the uncertianty method
@@ -593,6 +1050,8 @@ elif args.reproject_perturbed == 'fit-boards-ref':
     reproject_perturbed = reproject_perturbed__fit_boards_ref
 elif args.reproject_perturbed == 'diff':
     reproject_perturbed = reproject_perturbed__diff
+elif args.reproject_perturbed == 'optimize-cross-reprojection-error':
+    reproject_perturbed = reproject_perturbed__optimize_cross_reprojection_error
 else:
     raise Exception("getting here is a bug")
 
@@ -611,11 +1070,16 @@ for distance in args.distances:
                             extrinsics_baseline_mounted,
                             frames_baseline,
                             calobject_warp_baseline,
+                            optimization_inputs_baseline,
 
                             intrinsics_true,
                             extrinsics_true_mounted,
                             frames_true,
-                            calobject_warp_true)
+                            calobject_warp_true,
+                            # optimization_inputs_sampled not available here:
+                            # the "true" values aren't the result of an
+                            # optimization
+                            None)
 
     # I check the bias for cameras 0,1. Cameras 2,3 have q0 outside of the
     # chessboard region, or right on its edge, so regularization DOES affect
@@ -704,7 +1168,8 @@ if not args.do_sample:
 ( intrinsics_sampled,         \
   extrinsics_sampled_mounted, \
   frames_sampled,             \
-  calobject_warp_sampled ) =  \
+  calobject_warp_sampled,     \
+  optimization_inputs_sampled ) = \
       calibration_sample( args.Nsamples, args.Ncameras, args.Nframes,
                           Nintrinsics,
                           optimization_inputs_baseline,
@@ -740,11 +1205,13 @@ def check_uncertainties_at(q0_baseline, idistance):
                             extrinsics_baseline_mounted,
                             frames_baseline,
                             calobject_warp_baseline,
+                            optimization_inputs_baseline,
 
                             intrinsics_sampled,
                             extrinsics_sampled_mounted,
                             frames_sampled,
-                            calobject_warp_sampled)
+                            calobject_warp_sampled,
+                            optimization_inputs_sampled)
 
     # shape (Ncameras, 2)
     q_sampled_mean = np.mean(q_sampled, axis=-3)
@@ -854,7 +1321,7 @@ if args.show_distribution:
         data_tuples, plot_options = make_plot(icam)
 
         if args.extra_observation_at is not None:
-            plot_options['title'] += f': extra at {args.extra_observation_at}'
+            plot_options['title'] += f': boards at {args.range_to_boards}m, extra one at {args.extra_observation_at}m'
 
         plot_distribution[icam] = gp.gnuplotlib(**plot_options)
         plot_distribution[icam].plot(*data_tuples)
